@@ -1,6 +1,6 @@
-;;; shimbun.el --- interfacing with web newspapers
+;;; shimbun.el --- interfacing with web newspapers -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001 Yuuichi Teranishi <teranisi@gohome.org>
+;; Copyright (C) 2001, 2002, 2003 Yuuichi Teranishi <teranisi@gohome.org>
 
 ;; Author: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;         Akihiro Arisawa    <ari@mbf.sphere.ne.jp>,
@@ -32,7 +32,9 @@
 ;; Shimbun API:
 ;;
 ;; shimbun-open
+;; shimbun-server
 ;; shimbun-groups
+;; shimbun-current-group
 ;; shimbun-open-group
 ;; shimbun-close-group
 ;; shimbun-headers
@@ -65,6 +67,8 @@
 ;; shimbun-header-extra
 ;; shimbun-header-set-extra
 
+;;; Code:
+
 (eval-when-compile (require 'cl))
 (eval-when-compile (require 'static))
 
@@ -72,6 +76,7 @@
 (require 'eword-encode)
 (require 'luna)
 (require 'std11)
+(require 'shimbun-servers)
 
 (eval-and-compile
   (luna-define-class shimbun ()
@@ -79,7 +84,7 @@
 			  x-face x-face-alist
 			  url coding-system from-address
 			  content-start content-end
-			  expiration-days))
+			  expiration-days server-name))
   (luna-define-internal-accessors 'shimbun))
 
 (defgroup shimbun nil
@@ -102,6 +107,10 @@
 (luna-define-generic shimbun-mua-search-id (mua id)
   "Return non-nil when MUA found a message structure which corresponds to ID.")
 
+(defun shimbun-mua-shimbun (mua)
+  "Return the shimbun object created by MUA."
+  (shimbun-mua-shimbun-internal mua))
+
 ;;; BASE 64
 (require 'mel)
 (eval-and-compile
@@ -121,6 +130,20 @@ Return content-type of URL as string when retrieval succeeded."
 	  (goto-char (point-min))))
       type)))
 
+(defun shimbun-fetch-url (shimbun url &optional no-cache no-decode)
+  "Retrieve contents specified by URL for SHIMBUN.
+This function is exacly similar to `shimbun-retrieve-url', but
+considers the `coding-system' slot of SHIMBUN when estimating a coding
+system of retrieved contents."
+  (if (shimbun-coding-system-internal shimbun)
+      (let ((w3m-coding-system-priority-list
+	     (cons (shimbun-coding-system-internal shimbun)
+		   w3m-coding-system-priority-list)))
+	(inline
+	  (shimbun-retrieve-url url no-cache no-decode)))
+    (inline
+      (shimbun-retrieve-url url no-cache no-decode))))
+
 (defalias 'shimbun-content-type 'w3m-content-type)
 (defsubst shimbun-url-exists-p (url &optional no-cache)
   (string= "text/html"
@@ -135,67 +158,61 @@ Return content-type of URL as string when retrieval succeeded."
 				      extra)
   (vector number subject from date id references chars lines xref extra))
 
-;;(defsubst shimbun-header-number (header)
+;;(defun shimbun-header-number (header)
 ;;  (aref header 0))
 
-(defsubst shimbun-header-field-value ()
-  (let ((pt (point)))
-    (prog1
-	(buffer-substring (match-end 0) (std11-field-end))
-      (goto-char pt))))
-
-(defsubst shimbun-header-subject (header)
+(defun shimbun-header-subject (header)
   (aref header 1))
 
-(defsubst shimbun-header-set-subject (header subject)
+(defun shimbun-header-set-subject (header subject)
   (aset header 1 subject))
 
-(defsubst shimbun-header-from (header)
+(defun shimbun-header-from (header)
   (aref header 2))
 
-(defsubst shimbun-header-set-from (header from)
+(defun shimbun-header-set-from (header from)
   (aset header 2 from))
 
-(defsubst shimbun-header-date (header)
+(defun shimbun-header-date (header)
   (aref header 3))
 
-(defsubst shimbun-header-set-date (header date)
+(defun shimbun-header-set-date (header date)
   (aset header 3 date))
 
-(defsubst shimbun-header-id (header)
+(defun shimbun-header-id (header)
   (aref header 4))
 
-(defsubst shimbun-header-set-id (header id)
+(defun shimbun-header-set-id (header id)
   (aset header 4 id))
 
-(defsubst shimbun-header-references (header)
+(defun shimbun-header-references (header)
   (aref header 5))
 
-(defsubst shimbun-header-set-references (header references)
+(defun shimbun-header-set-references (header references)
   (aset header 5 references))
 
-(defsubst shimbun-header-chars (header)
+(defun shimbun-header-chars (header)
   (aref header 6))
 
-(defsubst shimbun-header-set-chars (header chars)
+(defun shimbun-header-set-chars (header chars)
   (aset header 6 chars))
 
-(defsubst shimbun-header-lines (header)
+(defun shimbun-header-lines (header)
   (aref header 7))
 
-(defsubst shimbun-header-set-lines (header lines)
+(defun shimbun-header-set-lines (header lines)
   (aset header 7 lines))
 
-(defsubst shimbun-header-xref (header)
+(defun shimbun-header-xref (header)
   (aref header 8))
 
-(defsubst shimbun-header-set-xref (header xref)
+(defun shimbun-header-set-xref (header xref)
   (aset header 8 xref))
 
-(defsubst shimbun-header-extra (header)
+(defun shimbun-header-extra (header)
   (aref header 9))
 
-(defsubst shimbun-header-set-extra (header extra)
+(defun shimbun-header-set-extra (header extra)
   (aset header 9 extra))
 
 ;; Inline functions for the internal use.
@@ -249,7 +266,8 @@ content-type if `shimbun-encapsulate-images' is non-nil."
 					 (set-buffer-multibyte nil)
 					 (setq
 					  type
-					  (shimbun-retrieve-url
+					  (shimbun-fetch-url
+					   shimbun
 					   (shimbun-expand-url
 					    url
 					    (shimbun-header-xref header))
@@ -303,7 +321,8 @@ content-type if `shimbun-encapsulate-images' is non-nil."
       (insert "<html>\n<head>\n<base href=\""
 	      (shimbun-header-xref header) "\">\n</head>\n<body>\n")
       (goto-char (point-max))
-      (insert "\n</body>\n</html>\n"))
+      (insert (shimbun-footer shimbun header t)
+	      "\n</body>\n</html>\n"))
     (shimbun-make-mime-article shimbun header)
     (buffer-string)))
 
@@ -367,15 +386,15 @@ set this to `never' if you never want to use BBDB."
 ;;; Implementation of Shimbun API.
 
 (defconst shimbun-attributes
-  '(url groups coding-system from-address content-start content-end
-	x-face-alist expiration-days))
+  '(url groups coding-system server-name from-address
+	content-start content-end x-face-alist expiration-days))
 
 (defun shimbun-open (server &optional mua)
   "Open a shimbun for SERVER.
 Optional MUA is a `shimbun-mua' instance."
   (require (intern (concat "sb-" server)))
-  (let (url groups coding-system from-address content-start content-end
-	    x-face-alist shimbun expiration-days)
+  (let (url groups coding-system server-name from-address
+	    content-start content-end x-face-alist shimbun expiration-days)
     (dolist (attr shimbun-attributes)
       (set attr
 	   (symbol-value (intern-soft
@@ -383,6 +402,7 @@ Optional MUA is a `shimbun-mua' instance."
     (setq shimbun (luna-make-entity (intern (concat "shimbun-" server))
 				    :mua mua
 				    :server server
+				    :server-name server-name
 				    :url url
 				    :groups groups
 				    :coding-system coding-system
@@ -395,12 +415,33 @@ Optional MUA is a `shimbun-mua' instance."
       (shimbun-mua-set-shimbun-internal mua shimbun))
     shimbun))
 
+(defun shimbun-server (shimbun)
+  "Return the server name of SHIMBUN."
+  (shimbun-server-internal shimbun))
+
+(luna-define-generic shimbun-server-name (shimbun)
+  "Return the server name of SHIMBUN in human-readable style.")
+
+(luna-define-method shimbun-server-name ((shimbun shimbun))
+  (or (shimbun-server-name-internal shimbun)
+      (shimbun-server-internal shimbun)))
+
 (luna-define-generic shimbun-groups (shimbun)
   "Return a list of groups which are available in the SHIMBUN.")
 
 (luna-define-method shimbun-groups ((shimbun shimbun))
   (shimbun-groups-internal shimbun))
 
+(defun shimbun-current-group (shimbun)
+  "Return the current group of SHIMBUN."
+  (shimbun-current-group-internal shimbun))
+
+(luna-define-generic shimbun-current-group-name (shimbun)
+  "Return the current group name of SHIMBUN in human-readable style.")
+
+(luna-define-method shimbun-current-group-name ((shimbun shimbun))
+  (shimbun-current-group-internal shimbun))
+
 (defun shimbun-open-group (shimbun group)
   "Open a SHIMBUN GROUP."
   (if (member group (shimbun-groups shimbun))
@@ -428,7 +469,7 @@ Return nil if all pages should be retrieved."
 
 (luna-define-method shimbun-headers ((shimbun shimbun) &optional range)
   (with-temp-buffer
-    (shimbun-retrieve-url (shimbun-index-url shimbun) 'reload)
+    (shimbun-fetch-url shimbun (shimbun-index-url shimbun) 'reload)
     (shimbun-get-headers shimbun range)))
 
 (luna-define-generic shimbun-reply-to (shimbun)
@@ -459,6 +500,16 @@ Return nil if all pages should be retrieved."
 Return nil when articles are not expired."
   (shimbun-expiration-days-internal shimbun))
 
+(luna-define-generic shimbun-from-address (shimbun)
+  "Make a From address like \"SERVER (GROUP) <ADDRESS>\".")
+
+(luna-define-method shimbun-from-address ((shimbun shimbun))
+  (format "%s (%s) <%s>"
+	  (shimbun-mime-encode-string (shimbun-server-name shimbun))
+	  (shimbun-mime-encode-string (shimbun-current-group-name shimbun))
+	  (or (shimbun-from-address-internal shimbun)
+	      (shimbun-reply-to shimbun))))
+
 (luna-define-generic shimbun-article (shimbun header &optional outbuf)
   "Retrieve a SHIMBUN article which corresponds to HEADER to the OUTBUF.
 HEADER is a shimbun-header which is obtained by `shimbun-headers'.
@@ -469,7 +520,7 @@ If OUTBUF is not specified, article is retrieved to the current buffer.")
     (with-current-buffer (or outbuf (current-buffer))
       (w3m-insert-string
        (or (with-temp-buffer
-	     (shimbun-retrieve-url (shimbun-article-url shimbun header))
+	     (shimbun-fetch-url shimbun (shimbun-article-url shimbun header))
 	     (message "shimbun: Make contents...")
 	     (goto-char (point-min))
 	     (prog1 (shimbun-make-contents shimbun header)
@@ -483,21 +534,12 @@ HEADER is a header structure obtained via `shimbun-headers'.")
 (luna-define-method shimbun-make-contents ((shimbun shimbun) header)
   (shimbun-make-html-contents shimbun header))
 
-(defun shimbun-header-insert-and-buffer-string (shimbun header &optional charset html)
-  "Insert headers which are generated from SHIMBUN and HEADER, and
-return the contents of this buffer as an encoded string."
-  (unless charset (setq charset "ISO-2022-JP"))
-  (goto-char (point-min))
-  (shimbun-header-insert shimbun header)
-  (insert "Content-Type: text/" (if html "html" "plain") "; charset=" charset
-	  "\nMIME-Version: 1.0\n\n")
-  (when html
-    (insert "<html><head><base href=\"" (shimbun-header-xref header)
-	    "\"></head><body>")
-    (goto-char (point-max))
-    (insert "</body></html>"))
-  (encode-coding-string (buffer-string)
-			(mime-charset-to-coding-system charset)))
+(luna-define-generic shimbun-footer (shimbun header &optional html)
+  "Make a footer string for SHIMBUN and HEADER.")
+
+(luna-define-method shimbun-footer ((shimbun shimbun) header &optional html)
+  "Return a null string for backends that have no footer."
+  "")
 
 (luna-define-generic shimbun-index-url (shimbun)
   "Return a index URL of SHIMBUN.")
@@ -515,13 +557,54 @@ integer n:    Retrieve n pages of header indices.")
 (luna-define-method shimbun-close ((shimbun shimbun))
   (shimbun-close-group shimbun))
 
+;;; Virtual class for Japanese Newspapers:
+(luna-define-class shimbun-japanese-newspaper () ())
+(luna-define-method shimbun-footer ((shimbun shimbun-japanese-newspaper) header
+				    &optional html)
+  (if html
+      (concat "\n<p align=\"left\">\n-- <br>\n$B$3$N5-;v$NCx:n8"$O!"(B"
+	      (shimbun-server-name shimbun)
+	      "$B<R$K5"B0$7$^$9!#(B\n$B86J*$O(B <a href=\""
+	      (shimbun-header-xref header) "\">"
+	      (shimbun-header-xref header)
+	      "</a> $B$G8x3+$5$l$F$$$^$9!#(B\n</p>\n")
+    (concat "\n-- \n$B$3$N5-;v$NCx:n8"$O!"(B"
+	    (shimbun-server-name shimbun)
+	    "$B<R$K5"B0$7$^$9!#(B\n$B86J*$O(B "
+	    (shimbun-header-xref header)
+	    " $B$G8x3+$5$l$F$$$^$9!#(B\n")))
+
 ;;; Misc Functions
+(defun shimbun-header-insert-and-buffer-string (shimbun header
+							&optional charset html)
+  "Insert headers which are generated from SHIMBUN and HEADER, and
+return the contents of this buffer as an encoded string."
+  (unless charset
+    (setq charset "ISO-2022-JP"))
+  (goto-char (point-min))
+  (shimbun-header-insert shimbun header)
+  (insert "Content-Type: text/" (if html "html" "plain") "; charset=" charset
+	  "\nMIME-Version: 1.0\n\n")
+  (if html
+      (progn
+	(insert "<html><head><base href=\"" (shimbun-header-xref header)
+		"\"></head><body>")
+	(goto-char (point-max))
+	(insert (shimbun-footer shimbun header html)
+		"\n</body></html>"))
+    (goto-char (point-max))
+    (insert (shimbun-footer shimbun header html)))
+  (encode-coding-string (buffer-string)
+			(mime-charset-to-coding-system charset)))
+
 (defun shimbun-mime-encode-string (string)
-  (mapconcat
-   #'identity
-   (split-string (or (eword-encode-string
-		      (shimbun-decode-entities-string string)) ""))
-   " "))
+  (condition-case nil
+      (mapconcat
+       #'identity
+       (split-string (or (eword-encode-string
+			  (shimbun-decode-entities-string string)) ""))
+       " ")
+    (error string)))
 
 (defun shimbun-make-date-string (year month day &optional time timezone)
   (format "%02d %s %04d %s %s"
