@@ -1,6 +1,6 @@
 ;;; w3mhack.el --- a hack to setup the environment for building w3m
 
-;; Copyright (C) 2001, 2002 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2001, 2002, 2003 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Author: Katsumi Yamaoka <yamaoka@jpl.org>
 ;; Keywords: w3m, WWW, hypermedia
@@ -66,6 +66,13 @@ nil means that all icons are installed to the default directory.")
 but do not execute them.")
 
 (require 'cl)
+
+;; This file called from Makefile with "/bin/sh" on environment
+;; variable of "SHELL".
+;; However not necessarily have "/bin/sh" in win32 environment.
+(when (eq system-type 'windows-nt)
+  (setq shell-file-name "cmdproxy.exe"))
+
 (unless (dolist (var nil t))
   ;; Override the macro `dolist' which may have been defined in egg.el.
   (load "cl-macs" nil t))
@@ -200,6 +207,11 @@ Error: You have to install APEL before building emacs-w3m, see manuals.
 
 (defconst shimbun-module-directory "shimbun")
 
+(defconst shimbun-servers-file "shimbun-servers.el"
+  "A file contains a list of all the shimbun servers.  It should be
+created under the `shimbun-module-directory' directory when building
+emacs-w3m.")
+
 (defconst w3mhack-colon-keywords-file "w3m-kwds.el")
 
 ;; Needed for interdependencies among w3m and shimbun modules.
@@ -225,7 +237,7 @@ Error: You have to install APEL before building emacs-w3m, see manuals.
 				     "w3m-fsf.el" "w3m-om.el" "w3m-xmas.el"))
 	 (ignores;; modules not to be byte-compiled.
 	  (append
-	   (list "w3mhack.el" w3mhack-colon-keywords-file)
+	   (list "w3mhack.el" "w3m-setup.el" w3mhack-colon-keywords-file)
 	   (w3mhack-mdelete (cond ((featurep 'xemacs)
 				   "w3m-xmas.el")
 				  ((boundp 'MULE)
@@ -261,11 +273,18 @@ Error: You have to install APEL before building emacs-w3m, see manuals.
 	    (setq modules (nconc modules (list (concat shimbun-dir file)))))
 	  ;; mew-shimbun check
 	  (unless (locate-library "mew-nntp")
-	    (push (concat shimbun-dir "mew-shimbun.el") ignores)))
+	    (push (concat shimbun-dir "mew-shimbun.el") ignores))
+	  ;; nnshimbun check
+	  (unless (let ((gnus (locate-library "gnus")))
+		    (when gnus
+		      (let ((load-path (list (file-name-directory gnus))))
+			(locate-library "nnimap"))))
+	    (push (concat shimbun-dir "nnshimbun.el") ignores)))
       (push "mime-w3m.el" ignores)
       (push "octet.el" ignores))
     (unless (featurep 'mule)
-      (push "w3m-weather.el" ignores))
+      (push "w3m-weather.el" ignores)
+      (push "w3m-symbol.el" ignores))
     (if (and (not (featurep 'xemacs))
 	     (<= w3mhack-emacs-major-version 20)
 	     (locate-library "bitmap"))
@@ -338,7 +357,7 @@ Error: You have to install APEL before building emacs-w3m, see manuals.
 	      (unless (file-directory-p dir)
 		(message "mkdir %s" dir)
 		(unless w3mhack-nonunix-dryrun
-		  (make-directory dir))))
+		  (make-directory dir 'parents))))
        (install (srcdir dstdir pattern)
 		(dolist (src (directory-files srcdir t pattern))
 		  (let ((dst (expand-file-name
@@ -409,7 +428,7 @@ Error: You have to install APEL before building emacs-w3m, see manuals.
 						  (match-end num))))
 			   (map-extents (lambda (extent maparg)
 					  (delete-extent extent))
-					string 0 (lenght string))
+					string 0 (length string))
 			   string)))))
 	       (string
 		(` (let ((num (, num)))
@@ -457,6 +476,7 @@ Examples of the optimization:
 "
     (let ((obsoletes '(file-coding-system
 		       file-coding-system-for-read
+		       pathname-coding-system
 		       pop-up-frame-alist))
 	  (varlist (copy-sequence (cadr form)))
 	  obsolete elements element value)
@@ -514,42 +534,68 @@ to remove some obsolete variables in the first argument VARLIST."
 
   (defun w3mhack-make-package ()
     "Make some files in the XEmacs package directory."
-    (let* ((package-dir (pop command-line-args-left))
+    (let* ((temp-dir (expand-file-name
+		      ;; A dir name must be ended with "/w3m".
+		      "w3m"
+		      (make-temp-name (expand-file-name "pkgtmp"))))
+	   (make-hardlink 'add-name-to-file)
+	   (package-dir (pop command-line-args-left))
 	   (lisp-dir (expand-file-name "lisp/w3m/" package-dir))
 	   (custom-load (expand-file-name "custom-load.el" lisp-dir))
 	   (generated-autoload-file (expand-file-name "auto-autoloads.el"
-						      lisp-dir))
-	   (els (nconc (directory-files default-directory nil "^[^#]+\\.el$")
+						      temp-dir))
+	   (els (nconc (directory-files default-directory nil "^[^#]+\\.el\\'")
 		       (directory-files (expand-file-name
 					 shimbun-module-directory)
-					nil "^[^#]+\\.el$")))
+					nil "^[^#]+\\.el\\'")))
 	   (elcs (with-temp-buffer
 		   (let ((standard-output (current-buffer)))
 		     (w3mhack-examine-modules)
 		     (split-string (buffer-string) " \\(shimbun/\\)?"))))
 	   (icons (directory-files (expand-file-name "icons/") nil
-				   "^[^#]+\\.xpm$"))
+				   "^[^#]+\\.xpm\\'"))
+	   (infos (directory-files (expand-file-name "doc/") nil
+				   "^[^#]+\\.info\\(-[0-9]+\\)?\\'"))
 	   (si:message (symbol-function 'message))
-	   manifest make-backup-files noninteractive)
-      (when (file-exists-p custom-load)
-	(delete-file custom-load))
-      (when (file-exists-p (concat custom-load "c"))
-	(delete-file (concat custom-load "c")))
+	   hardlink manifest make-backup-files noninteractive)
+      ;; Non-Mule XEmacs cannot handle .el files containing non-ascii chars.
+      ;; So, we should make custom-load.el and auto-autoloads.el for only
+      ;; .el files which have been byte-compiled.  The following processings
+      ;; are performed in the `temp-dir' only to those files.
+      (make-directory temp-dir t)
+      (unless (condition-case nil
+		  (progn
+		    (setq hardlink (expand-file-name "w3m.el" temp-dir))
+		    (funcall make-hardlink "w3m.el" hardlink)
+		    (prog1
+			(file-exists-p hardlink)
+		      (delete-file hardlink)))
+		(error nil))
+	;; This system doesn't allow hard links.
+	(setq make-hardlink 'copy-file))
+      (dolist (el (nconc (directory-files default-directory t "\\.elc\\'")
+			 (directory-files "shimbun" t "\\.elc\\'")))
+	(setq el (substring el 0 -1))
+	(funcall make-hardlink
+		 el (expand-file-name (file-name-nondirectory el) temp-dir)))
       (with-temp-buffer
 	(let ((standard-output (current-buffer)))
-	  (Custom-make-dependencies lisp-dir))
+	  (Custom-make-dependencies temp-dir))
 	;; Print messages into stderr.
 	(message "%s" (buffer-string)))
-      (when (file-exists-p custom-load)
-	(require 'cus-load)
-	(byte-compile-file custom-load)
-	(push "custom-load.el" els)
-	(push "custom-load.elc" elcs))
+      (if (file-exists-p (expand-file-name "custom-load.el" temp-dir))
+	  (progn
+	    (copy-file (expand-file-name "custom-load.el" temp-dir)
+		       custom-load t)
+	    (require 'cus-load)
+	    (byte-compile-file custom-load)
+	    (push "custom-load.el" els)
+	    (push "custom-load.elc" elcs))
+	(when (file-exists-p custom-load)
+	  (delete-file custom-load))
+	(when (file-exists-p (concat custom-load "c"))
+	  (delete-file (concat custom-load "c"))))
       (message "Updating autoloads for the directory %s..." lisp-dir)
-      (when (file-exists-p generated-autoload-file)
-	(delete-file generated-autoload-file))
-      (when (file-exists-p (concat generated-autoload-file "c"))
-	(delete-file (concat generated-autoload-file "c")))
       (defun message (fmt &rest args)
 	"Ignore useless messages while generating autoloads."
 	(cond ((and (string-equal "Generating autoloads for %s..." fmt)
@@ -560,26 +606,42 @@ to remove some obsolete variables in the first argument VARLIST."
 	      ((string-equal "Generating autoloads for %s...done" fmt))
 	      (t (apply si:message fmt args))))
       (unwind-protect
-	  (update-autoloads-from-directory lisp-dir)
+	  (update-autoloads-from-directory temp-dir)
 	(fset 'message si:message))
-      (when (file-exists-p generated-autoload-file)
-	(byte-compile-file generated-autoload-file)
-	(push "auto-autoloads.el" els)
-	(push "auto-autoloads.elc" elcs))
+      (if (file-exists-p generated-autoload-file)
+	  (progn
+	    (copy-file generated-autoload-file
+		       (expand-file-name "auto-autoloads.el" lisp-dir) t)
+	    (byte-compile-file (expand-file-name "auto-autoloads.el" lisp-dir))
+	    (push "auto-autoloads.el" els)
+	    (push "auto-autoloads.elc" elcs))
+	(setq generated-autoload-file (expand-file-name "auto-autoloads.el"
+							lisp-dir))
+	(when (file-exists-p generated-autoload-file)
+	  (delete-file generated-autoload-file))
+	(when (file-exists-p (concat generated-autoload-file "c"))
+	  (delete-file (concat generated-autoload-file "c"))))
+      ;; Clear the `temp-dir'.
+      (dolist (el (directory-files temp-dir t "\\.el\\'"))
+	(delete-file el))
+      (delete-directory temp-dir)
+      (delete-directory (expand-file-name ".." temp-dir))
       (when (file-directory-p (expand-file-name "pkginfo/" package-dir))
 	(setq manifest (expand-file-name "pkginfo/MANIFEST.w3m" package-dir))
 	(message "Generating %s..." manifest)
 	(with-temp-file manifest
 	  (insert "pkginfo/MANIFEST.w3m\n")
 	  (dolist (log (directory-files lisp-dir nil
-					"^ChangeLog\\(\\.[0-9]+\\)?$"))
+					"^ChangeLog\\(\\.[0-9]+\\)?\\'"))
 	    (insert "lisp/w3m/" log "\n"))
 	  (dolist (el els)
 	    (insert "lisp/w3m/" el "\n")
 	    (when (member (concat el "c") elcs)
 	      (insert "lisp/w3m/" el "c\n")))
 	  (dolist (icon icons)
-	    (insert "etc/w3m/" icon "\n")))
+	    (insert "etc/w3m/" icon "\n"))
+	  (dolist (info infos)
+	    (insert "info/" info "\n")))
 	(message "Generating %s...done" manifest)))))
 
  ((>= w3mhack-emacs-major-version 21)
@@ -612,6 +674,96 @@ to remove some obsolete variables in the first argument VARLIST."
 		  (, (append '(funcall fn) (cdr form)))
 		(fset 'message msg))))))))
 
+(defun w3mhack-generate-shimbun-servers-file ()
+  "Generate a file which contains a list of all the shimbun servers.
+The file name and the directory are specified by `shimbun-servers-file'
+and `shimbun-module-directory'."
+  (let ((srvr-file (expand-file-name shimbun-servers-file
+				     shimbun-module-directory))
+	(servers (sort
+		  (delq
+		   nil
+		   (mapcar
+		    (lambda (file)
+		      (let ((server (substring file 3 -3)))
+			(unless (member server
+					'("fml" "glimpse" "lump" "mailarc"
+					  "mailman" "mhonarc" "text"))
+			  server)))
+		    (directory-files shimbun-module-directory nil
+				     "^sb-.*\\.el$")))
+		  'string-lessp))
+	(buffer (get-buffer-create " *shimbun servers*"))
+	shimbun-servers-list buffer-file-format format-alist
+	insert-file-contents-post-hook insert-file-contents-pre-hook)
+    (save-excursion
+      (set-buffer buffer)
+      (erase-buffer)
+      (unless (and (file-exists-p srvr-file)
+		   (progn
+		     (insert-file-contents srvr-file)
+		     (goto-char (point-min))
+		     (and (re-search-forward
+			   "^(defconst shimbun-servers-list$" nil t)
+			  (goto-char (match-beginning 0))
+			  (condition-case nil
+			      (eval (read (current-buffer)))
+			    (error nil))
+			  (setq shimbun-servers-list
+				(sort shimbun-servers-list 'string-lessp))
+			  (equal servers shimbun-servers-list))))
+	(when (file-exists-p srvr-file)
+	  (delete-file srvr-file))
+	(when (file-exists-p (concat srvr-file "c"))
+	  (delete-file (concat srvr-file "c")))
+	(erase-buffer)
+	(insert "\
+;;; " shimbun-servers-file " --- variable setting for shimbun.el
+
+;; This file should be generated by make in emacs-w3m source directory.
+;; Regulation of copyright is declared according to the description of
+;; shimbun.el.
+
+\(defvar shimbun-additional-servers-list nil
+  \"*A list of the user-defined shimbun servers.  For instance, if you use
+the \\\"sb-foo.el\\\" module and the \\\"sb-bar.el\\\" module for your own purpose,
+you can add those server names to this variable as follows:
+
+\\(setq shimbun-additional-servers-list '(\\\"foo\\\" \\\"bar\\\"))\")
+
+\(defconst shimbun-servers-list
+  '(")
+	(let (server)
+	  (while servers
+	    (setq server (prin1-to-string (pop servers)))
+	    (when (> (+ (current-column) (length server)) (if servers
+							      79
+							    77))
+	      (delete-char -1)
+	      (insert "\n    "))
+	    (insert server " ")))
+	(delete-char -1)
+	(insert "))
+
+\(defun shimbun-servers-list ()
+  (append shimbun-servers-list shimbun-additional-servers-list))
+
+\(defun shimbun-servers-alist ()
+  (mapcar 'list (shimbun-servers-list)))
+
+\(provide '" (file-name-sans-extension shimbun-servers-file) ")
+
+;;; " shimbun-servers-file " ends here
+")
+	(write-region (point-min) (point-max) srvr-file)))
+    (kill-buffer buffer)))
+
+(let ((dir (expand-file-name "../" shimbun-module-directory)))
+  (when (and (file-writable-p dir)
+	     (file-exists-p (expand-file-name "Makefile.in" dir)))
+    (byte-compile 'w3mhack-generate-shimbun-servers-file)
+    (w3mhack-generate-shimbun-servers-file)))
+
 (defun w3mhack-generate-colon-keywords-file ()
   "Generate a file which contains a list of colon keywords to be bound at
 run-time.  The file name is specified by `w3mhack-colon-keywords-file'."
@@ -700,7 +852,8 @@ run-time.  The file name is specified by `w3mhack-colon-keywords-file'."
 				    (read buffer)
 				  (error nil)))
 		(while form
-		  (setq elem (pop form))
+		  (setq elem (car-safe form)
+			form (cdr-safe form))
 		  (unless (memq (car-safe elem)
 				'(\` backquote defcustom defface defgroup
 				  define-widget quote))
@@ -714,10 +867,8 @@ run-time.  The file name is specified by `w3mhack-colon-keywords-file'."
 			       (not (memq elem ignores))
 			       (not (memq elem keywords)))
 		      (push elem keywords)))))))
-	  (setq keywords (sort keywords
-			       (lambda (a b)
-				 (string-lessp (symbol-name a)
-					       (symbol-name b)))))
+	  ;; `string-lessp' allows symbols as well.
+	  (setq keywords (sort keywords 'string-lessp))
 	  (erase-buffer)
 	  (insert ";;; " w3mhack-colon-keywords-file "\
  --- List of colon keywords which will be bound at run-time
@@ -731,20 +882,32 @@ run-time.  The file name is specified by `w3mhack-colon-keywords-file'."
 		  "\n
 \(defvar w3m-colon-keywords)
 \(setq w3m-colon-keywords
-      '("
-		  (mapconcat 'symbol-name keywords "\n\t")
-		  "))\n")
+      '(" (symbol-name (pop keywords)) "\n\t")
+	  (let (keyword)
+	    (while keywords
+	      (setq keyword (symbol-name (pop keywords)))
+	      (when (> (+ (current-column) (length keyword)) (if keywords
+								 79
+							       77))
+		(delete-char -1)
+		(insert "\n\t"))
+	      (insert keyword " ")))
+	  (delete-char -1)
+	  (insert "))\n")
 	  (write-region (point-min) (point) kwds-file))))
     (kill-buffer buffer)))
 
-(condition-case nil
-    (let ((kwds-file (expand-file-name w3mhack-colon-keywords-file)))
-      :symbol-for-testing-whether-colon-keyword-is-available-or-not
+(let* ((kwds-file (expand-file-name w3mhack-colon-keywords-file))
+       (dir (file-name-directory kwds-file)))
+  (when (and (file-writable-p dir)
+	     (file-exists-p (expand-file-name "Makefile.in" dir)))
+    (condition-case nil
+	:symbol-for-testing-whether-colon-keyword-is-available-or-not
       (when (file-exists-p kwds-file)
-	(delete-file (expand-file-name w3mhack-colon-keywords-file))))
-  (void-variable
-   (byte-compile 'w3mhack-generate-colon-keywords-file)
-   (w3mhack-generate-colon-keywords-file)))
+	(delete-file (expand-file-name w3mhack-colon-keywords-file)))
+      (void-variable
+       (byte-compile 'w3mhack-generate-colon-keywords-file)
+       (w3mhack-generate-colon-keywords-file)))))
 
 (defun w3mhack-load-path ()
   "Print default value of additional load paths for w3m.el."
@@ -768,6 +931,8 @@ run-time.  The file name is specified by `w3mhack-colon-keywords-file'."
 	 (push (file-name-directory x) paths))
     (if (setq x (locate-library "mew"))
 	(push (file-name-directory x) paths))
+    (if (setq x (locate-library "gnus"))
+	(push (file-name-directory x) paths))
     (and (if (featurep 'xemacs)
 	     ;; Mule-UCS does not support XEmacs versions prior to 21.2.37.
 	     (and (>= emacs-major-version 21)
@@ -790,16 +955,20 @@ run-time.  The file name is specified by `w3mhack-colon-keywords-file'."
   "Show what files should be installed and where should they go."
   (let ((lisp-dir (pop command-line-args-left))
 	(icon-dir (pop command-line-args-left))
-	(package-dir (pop command-line-args-left)))
+	(package-dir (pop command-line-args-left))
+	(info-dir (pop command-line-args-left)))
     (message "
 lispdir=%s
 ICONDIR=%s
-PACKAGEDIR=%s"
-	     lisp-dir icon-dir package-dir)
+PACKAGEDIR=%s
+infodir=%s"
+	     lisp-dir icon-dir package-dir info-dir)
+    (setq info-dir (file-name-as-directory info-dir))
     (message "
 install:
-  *.el, *.elc, ChangeLog* -> %s"
-	     (file-name-as-directory lisp-dir))
+  *.el, *.elc, ChangeLog* -> %s
+  *.info, *.info-*        -> %s"
+	     (file-name-as-directory lisp-dir) info-dir)
     (setq icon-dir (file-name-as-directory icon-dir))
     (unless (string-equal "NONE/" icon-dir)
       (message "
@@ -807,15 +976,135 @@ install-icons:
   *.xpm                   -> %s"
 	       icon-dir))
     (setq package-dir (file-name-as-directory package-dir))
+    (message "
+install-info:
+  *.info, *.info-*        -> %s"
+	     info-dir)
     (unless (string-equal "NONE/" package-dir)
       (message "
 install-package:
   *.el, *.elc, ChangeLog* -> %slisp/w3m/
   *.xpm                   -> %setc/w3m/
+  *.info, *.info-*        -> %sinfo/
   MANIFEST.w3m            -> %spkginfo/"
-	       package-dir package-dir package-dir)))
+	       package-dir package-dir package-dir package-dir)))
   (message (if (featurep 'xemacs) "\n" "")))
 
+(defun w3mhack-makeinfo ()
+  "Emacs makeinfo in batch mode.
+NOTE: This function must be called from the top directory."
+  (load "doc/ptexinfmt.el" nil t t)
+  (cd "doc")
+  (let ((file (pop command-line-args-left))
+	auto-save-default
+	find-file-run-dired
+	coding-system-for-write
+	output-coding-system
+	(error 0))
+    (if (and (string-match "-ja\\.texi\\'" file)
+	     (not (featurep 'mule)))
+	(message "Cannot format %s with this version of %sEmacs (ignored)"
+		 file
+		 (if (featurep 'xemacs) "X" ""))
+      (condition-case err
+	  (progn
+	    (find-file file)
+	    (setq buffer-read-only nil)
+	    (buffer-disable-undo (current-buffer))
+	    (cond ((boundp 'MULE)
+		   (setq output-coding-system file-coding-system))
+		  ((boundp 'buffer-file-coding-system)
+		   (setq coding-system-for-write
+			 (symbol-value 'buffer-file-coding-system))))
+	    ;; Remove ignored areas first.
+	    (while (re-search-forward "^@ignore[\t\r ]*$" nil t)
+	      (delete-region (match-beginning 0)
+			     (if (re-search-forward
+				  "^@end[\t ]+ignore[\t\r ]*$" nil t)
+				 (1+ (match-end 0))
+			       (point-max))))
+	    ;; Remove unsupported commands.
+	    (goto-char (point-min))
+	    (while (re-search-forward "@\\(end \\)?ifnottex" nil t)
+	      (replace-match ""))
+	    (goto-char (point-min))
+	    (while (search-forward "\n@iflatex\n" nil t)
+	      (delete-region (1+ (match-beginning 0))
+			     (search-forward "\n@end iflatex\n")))
+	    (goto-char (point-min))
+	    ;; process @include before updating node
+	    ;; This might produce some problem if we use @lowersection or
+	    ;; such.
+	    (let ((input-directory default-directory)
+		  (texinfo-command-end))
+	      (while (re-search-forward "^@include" nil t)
+		(setq texinfo-command-end (point))
+		(let ((filename (concat input-directory
+					(texinfo-parse-line-arg))))
+		  (re-search-backward "^@include")
+		  (delete-region (point) (save-excursion
+					   (forward-line 1)
+					   (point)))
+		  (message "Reading included file: %s" filename)
+		  (save-excursion
+		    (save-restriction
+		      (narrow-to-region
+		       (point) (+ (point)
+				  (car (cdr (insert-file-contents filename)))))
+		      (goto-char (point-min))
+		      ;; Remove `@setfilename' line from included file,
+		      ;; if any, so @setfilename command not duplicated.
+		      (if (re-search-forward "^@setfilename"
+					     (save-excursion
+					       (forward-line 100)
+					       (point))
+					     t)
+			  (progn
+			    (beginning-of-line)
+			    (delete-region (point) (save-excursion
+						     (forward-line 1)
+						     (point))))))))))
+	    (texinfo-mode)
+	    (texinfo-every-node-update)
+	    (set-buffer-modified-p nil)
+	    (message "texinfo formatting %s..." file)
+	    (if (featurep 'mule)
+		;; Encode messages to terminal.
+		(let ((si:message (symbol-function 'message)))
+		  (fset 'message
+			(byte-compile
+			 (if (boundp 'MULE)
+			     (lambda (fmt &rest args)
+			       (funcall si:message "%s"
+					(code-convert-string
+					 (apply 'format fmt args)
+					 '*internal* '*junet*)))
+			   (lambda (fmt &rest args)
+			     (funcall si:message "%s"
+				      (encode-coding-string
+				       (apply 'format fmt args)
+				       'iso-2022-7bit))))))
+		  (unwind-protect
+		      (texinfo-format-buffer nil)
+		    (fset 'message si:message)))
+	      (texinfo-format-buffer nil))
+	    (if (buffer-modified-p)
+		(progn (message "Saving modified %s" (buffer-file-name))
+		       (save-buffer))))
+	(error
+	 (message ">> Error: %s" (prin1-to-string err))
+	 (message ">>  point at")
+	 (let ((s (buffer-substring (point) (min (+ (point) 100) (point-max))))
+	       (tem 0))
+	   (while (setq tem (string-match "\n+" s tem))
+	     (setq s (concat (substring s 0 (match-beginning 0))
+			     "\n>>  "
+			     (substring s (match-end 0)))
+		   tem (1+ tem)))
+	   (message ">>  %s" s))
+	 (setq error 1))))
+    (kill-emacs error)))
+
 (defun w3mhack-version ()
   "Print version of w3m.el."
   (require 'w3m)
